# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

try:
    import coverage
except ImportError:
    coverage = None

from django.conf import settings
from django.test.simple import run_tests as django_test_runner

def test_runner_with_coverage(test_labels, verbosity=1, interactive=True, extra_tests=[]):
    coverage_enabled = coverage and hasattr(settings, 'COVERAGE_MODULES')

    if coverage_enabled:
        coverage.use_cache(0)
        coverage.start()

    test_results = django_test_runner(test_labels, verbosity, interactive, extra_tests)

    if coverage_enabled:
        coverage.stop()
        coverage_modules = []
        for module in settings.COVERAGE_MODULES:
            coverage_modules.append(__import__(module, globals(), locals(), ['']))
        coverage.report(coverage_modules, show_missing=1)

    return test_results



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from django_zodb.utils import parse_uri

REQUIRED = True


def parse_bool(value):
    return value and value.lower()[0] not in 'nf0'


def parse_tuple(values):
    return tuple(value.strip() for value in values.split(","))


class Configuration(object):
    def __init__(self, uri):
        self.configuration = self._parse_uri(uri)

    def _parse_uri(self, uri):
        config = parse_uri(uri)
        query = config.pop('query', {})
        for key, values in query.items():
            if key in config:
                raise ValueError("Cannot override %r argument." % key)
            value = values[-1]  # only last argument
            if value == '':     # ?arg1&arg2&... == ?arg1=1&arg2=1&...
                config[key] = '1'
            else:
                config[key] = value
        return config

    # (uri_arg_name, type_, storage_arg_name, required)
    def get_settings(self, specs):
        ret = {}

        for spec in specs:

            if len(spec) == 3:
                spec = spec + (not REQUIRED,)

            key, type_, arg, required = spec
            try:
                value = self.configuration.pop(key)
                ret[arg] = type_(value)
            except KeyError:
                if required:
                    raise TypeError("Missing argument %r" % key)
            except (ValueError, TypeError):
                raise TypeError("Invalid argument %r" % key)

        return ret

    def pop(self, key, default=None):
        return self.configuration.pop(key, default)

    def get(self, key, default=None):
        return self.configuration.get(key, default)


def get_configuration_from_uri(uri):
    return Configuration(uri)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from django.test import TestCase

from test_models import ROOT, FakeContainer

class ViewsTests(TestCase):
    def tearDown(self):
        from django_zodb import views
        views.registry.clean()

    def eq(self, a, b, *args, **kwargs):
        return self.assertEquals(a, b, *args, **kwargs)

    def raise_(self, err, func, *args, **kwargs):
        return self.assertRaises(err, func, *args, **kwargs)

    def test_split_path(self):
        from django_zodb.views import split_path as sp
        self.eq(repr(sp("foo/bar/baz")), "(u'foo', u'bar', u'baz')")
        self.eq(repr(sp("/foo/bar/baz")), "(u'foo', u'bar', u'baz')")
        self.eq(repr(sp("foo/bar/baz/")), "(u'foo', u'bar', u'baz')")
        self.eq(repr(sp("/foo/bar/baz/")), "(u'foo', u'bar', u'baz')")
        self.eq(repr(sp("/foo/b%20r/baz/")), "(u'foo', u'b r', u'baz')")
        self.eq(repr(sp("/foo/b%20r///baz/")), "(u'foo', u'b r', u'baz')")
        self.eq(repr(sp("/foo/b%20r/./baz/")), "(u'foo', u'b r', u'baz')")
        self.eq(repr(sp("/foo/b%20r/./baz/.")), "(u'foo', u'b r', u'baz')")
        self.eq(repr(sp("/foo/b%20r/../baz/")), "(u'foo', u'baz')")
        self.eq(repr(sp("/foo/b%20r/../baz/..")), "(u'foo',)")
        self.eq(repr(sp("/foo/bar/b%C3%BDz/")), "(u'foo', u'bar', u'b\\xfdz')")
        self.raise_(TypeError, sp, "/foo/bar/b%C3%00z/")

    def test_traverse_result(self):
        from django_zodb.views import traverse as tr

        def _eq(res, **kw):
            self.eq(res.root, kw['root'], "root: %r != %r" % (res.root, kw['root']))
            self.eq(res.context, kw['ctx'], "ctx: %r != %r" % (res.context, kw['ctx']))
            self.eq(res.method_name, kw['mn'], "mn: %r != %r" % (res.method_name, kw['mn']))
            self.eq(type(res.method_name), type(kw['mn']))
            self.eq(res.subpath, kw['sp'], "sp: %r != %r" % (res.subpath, kw['sp']))
            self.eq(type(res.subpath), tuple)
            self.eq(res.traversed, kw['tr'], "tr: %r != %r" % (res.traversed, kw['tr']))
            self.eq(type(res.traversed), tuple)

        _eq(tr(ROOT, ""), root=ROOT, ctx=ROOT, mn=u"", sp=(), tr=())
        _eq(tr(ROOT, "foo"), root=ROOT, ctx=ROOT['foo'], mn=u"", sp=(), tr=(u'foo',))
        _eq(tr(ROOT, "/notfound/"), root=ROOT, ctx=ROOT, mn=u"notfound", sp=(), tr=(u'notfound',))
        _eq(tr(ROOT, "./foo"), root=ROOT, ctx=ROOT['foo'], mn=u"", sp=(), tr=(u'foo',))
        _eq(tr(ROOT, "/foo"), root=ROOT, ctx=ROOT['foo'], mn=u"", sp=(), tr=(u'foo',))
        _eq(tr(ROOT, "/foo/bar"), root=ROOT, ctx=ROOT['foo']['bar'], mn=u"", sp=(), tr=(u'foo', u'bar'))
        _eq(tr(ROOT, "/foo/bar/baz"), root=ROOT, ctx=ROOT['foo']['bar'], mn=u"baz", sp=(), tr=(u'foo', u'bar', u'baz'))
        _eq(tr(ROOT, "/foo/bar/baz/subpath/1/2"), root=ROOT, ctx=ROOT['foo']['bar'], mn=u"baz", sp=(u'subpath', u'1', u'2'), tr=(u'foo', u'bar', u'baz'))
        _eq(tr(ROOT, "/@@foo/bar/baz/"), root=ROOT, ctx=ROOT, mn=u"foo", sp=(u'bar', u'baz'), tr=(u'@@foo',))
        _eq(tr(ROOT, "/foo/@@bar/baz/"), root=ROOT, ctx=ROOT['foo'], mn=u"bar", sp=(u'baz',), tr=(u'foo', u'@@bar'))
        _eq(tr(ROOT, "/foo/bar/@@baz/subpath/1/2"), root=ROOT, ctx=ROOT['foo']['bar'], mn=u"baz", sp=(u'subpath', u'1', u'2'), tr=(u'foo', u'bar', u'@@baz'))
        _eq(tr(ROOT, "/foo/bar/baz/@@subpath/1/2"), root=ROOT, ctx=ROOT['foo']['bar'], mn=u"baz", sp=(u'@@subpath', u'1', u'2'), tr=(u'foo', u'bar', u'baz'))
        _eq(tr(ROOT, "/foo/qux/"), root=ROOT, ctx=ROOT['foo']['qux'], mn=u"", sp=(), tr=(u'foo', u'qux'))
        _eq(tr(ROOT, "/foo/qux/quxx"), root=ROOT, ctx=ROOT['foo']['qux']['quxx'], mn=u"", sp=(), tr=(u'foo', u'qux', u'quxx'))
        _eq(tr(ROOT, "/foo/qux/quxx/quxxx"), root=ROOT, ctx=ROOT['foo']['qux']['quxx'], mn=u"quxxx", sp=(), tr=(u'foo', u'qux', u'quxx', u'quxxx'))
        _eq(tr(ROOT, "/foo/qux/quxx/quxxx/sub"), root=ROOT, ctx=ROOT['foo']['qux']['quxx'], mn=u"quxxx", sp=(u'sub',), tr=(u'foo', u'qux', u'quxx', u'quxxx'))
        _eq(tr(ROOT['foo'], "/foo"), root=ROOT['foo'], ctx=ROOT['foo'], mn=u"foo", sp=(), tr=(u'foo',))

    def test_get_response_or_404_model(self):
        from django_zodb import views

        class _ContainerView(views.View):
            def __index__(self, request, context, root, subpath, traversed):
                return "__index__ response: %r" % request
        views.registry.register(model=FakeContainer, view=_ContainerView())

        response = views.get_response_or_404("request", ROOT, "/foo/bar")
        self.eq(response, "__index__ response: 'request'")

    def test_get_response_or_404_method_1(self):
        from django_zodb import views

        class _ContainerView(views.View):
            def __index__(self, request, context, root, subpath, traversed):
                return "__index__ response: %r" % request
            def baz(self, request, context, root, subpath, traversed):
                return "baz response: %r" % request
        views.registry.register(model=FakeContainer, view=_ContainerView())

        response = views.get_response_or_404("request", ROOT, "/foo/bar/baz")
        self.eq(response, "baz response: 'request'")

    def test_get_response_or_404_method_2(self):
        from django_zodb import views

        class _ContainerView(views.View):
            def __index__(self, request, context, root, subpath, traversed):
                return "__index__ response: %r" % request
            def baz(self, request, context, root, subpath, traversed):
                return "baz response: %r subpath: %r" % (request, "/".join(subpath))
        views.registry.register(model=FakeContainer, view=_ContainerView())

        response = views.get_response_or_404("request", ROOT, "/foo/bar/baz/subpath/1")
        self.eq(response, "baz response: 'request' subpath: u'subpath/1'")

    def test_fail_get_response_or_404_view_not_found(self):
        from django_zodb import views
        from django.http import Http404

        self.raise_(Http404, views.get_response_or_404, "request", ROOT, "/foo/bar")

    def test_fail_get_response_or_404_method_not_found(self):
        from django_zodb import views
        from django.http import Http404

        class _ContainerView(views.View):
            def __index__(self, request, context, root, subpath, traversed):
                return "__index__ response: %r" % request
        views.registry.register(model=FakeContainer, view=_ContainerView())

        self.assertRaises(Http404, views.get_response_or_404, "request", ROOT, "/foo/bar/baz")

    def test_view_results(self):
        from django_zodb import views

        t = views.TraverseResult(root="root")
        self.assertEquals(t.root, "root")
        self.assertEquals(t.method_name, u"")

        t['bla'] = 'ble'
        self.assertEquals(t.bla, 'ble')
        self.assertEquals(t['bla'], 'ble')

        t.foo = "bar"
        self.assertEquals(t.foo, 'bar')
        self.assertEquals(t['foo'], 'bar')

        self.assertEquals(sorted(t.items()), [
            ('bla', 'ble'),
            ('context', None),
            ('foo', 'bar'),
            ('method_name', u''),
            ('root', 'root'),
            ('subpath', ()),
            ('traversed', ())
        ])



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import os
import shutil
import subprocess
import logging


TOOLS_DIR = os.path.dirname(os.path.realpath(__file__))

def remove_db_files():
    filelist = (
        '/tmp/test.db',
        '/tmp/test.db.lock',
        '/tmp/test.db.index',
        '/tmp/test.db.tmp',
        '/tmp/blobdir',
    )
    for filename in filelist:
        if os.path.isdir(filename):
            shutil.rmtree(filename)
        if os.path.exists(filename):
            os.remove(filename)

def get_tool_path(filename):
    return os.path.join(TOOLS_DIR, filename)

def start_zeo(mode='host'):
    zeoconf = get_tool_path('zeo_test.conf')
    if mode == 'host':
        args = ["runzeo", '-C', zeoconf]
    else:
        args = ["runzeo", '-a', '/tmp/zeo.zdsock', '-C', zeoconf]
    zeo = subprocess.Popen(args)
    return zeo


class _NullHandler(logging.Handler):
    def emit(self, record):
        pass

def turn_off_log(logger):
    logging.getLogger(logger).addHandler(_NullHandler())




# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from django.conf.urls.defaults import *

urlpatterns = patterns('',
    (r'^(?P<path>.*)/?$', 'samples.wiki.views.page'),
)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from ZEO.ClientStorage import ClientStorage

from django_zodb.config import parse_bool
from django_zodb.storage.base import AbstractStorageFactory


class ZEOFactory(AbstractStorageFactory):
    _storage = ClientStorage
    _storage_args = (
        ('host', str, 'host'),
        ('port', int, 'port'),
        ('path', str, 'path'),
        ('storage', str, 'storage'),
        ('cache_size', int, 'cache_size'),
        ('name', str, 'name'),
        ('client', str, 'client'),
        ('var', str, 'var'),
        ('min_disconnect_poll', int, 'min_disconnect_poll'),
        ('max_disconnect_poll', int, 'max_disconnect_poll'),
        ('wait', parse_bool, 'wait'),
        ('wait_timeout', int, 'wait_timeout'),
        ('read_only', parse_bool, 'read_only'),
        ('read_only_fallback', parse_bool, 'read_only_fallback'),
        ('username', str, 'username'),
        ('password', str, 'password'),
        ('realm', str, 'realm'),
        ('blob_dir', str, 'blob_dir'),
        ('shared_blob_dir', parse_bool, 'shared_blob_dir'),
        ('drop_cache_rather_verify', parse_bool, 'drop_cache_rather_verify'),
        ('blob_cache_size', int, 'blob_cache_size'),
        ('blob_cache_size_check', int, 'blob_cache_size_check'),
    )

    def get_base_storage(self, **kwargs):
        host = kwargs.pop('host', None)
        port = kwargs.pop('port', 8090)
        path = kwargs.pop('path', None)
        if host and port:
            kwargs['addr'] = (host, port)
        elif path:
            kwargs['addr'] = path
        else:
            raise ValueError("Missing host:port address or path to socket to ZEO Server.")
        return self._storage(**kwargs)



# -*- coding: utf-8 -*-
#
# django-zodb sample - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

import re

from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect

import transaction
from django_zodb import views
from django_zodb import models

from samples.wiki.models import Wiki, Page
from samples.wiki.forms import PageEditForm

wikiwords = re.compile(ur"\b([A-Z]\w+([A-Z]+\w+)+)")


class WikiView(views.View):
    def __index__(self, request, context, root, subpath, traversed):
        return HttpResponseRedirect("FrontPage")

    def add(self, request, context, root, subpath, traversed):
        try:
            name = subpath[0]
        except IndexError:
            return HttpResponseRedirect("/wiki/")

        if request.method == "POST":
            form = PageEditForm(request.POST)
            if form.is_valid():
                page = Page(form.cleaned_data['content'])
                root[name] = page
                return HttpResponseRedirect(page.get_absolute_url())
        else:
            form = PageEditForm()

        page_data = {
            'name': name,
            'cancel_link': "javascript:history.go(-1)",
            'form': form,
        }
        return render_to_response("edit.html", page_data)
views.registry.register(model=Wiki, view=WikiView())


class PageView(views.View):
    def __index__(self, request, context, root, subpath, traversed):
        content = context.html()

        def check(match):
            word = match.group(1)
            if word in root:
                page = root[word]
                view_url = page.get_absolute_url()
                return '<a href="%s">%s</a>' % (view_url, word)
            else:
                add_url = models.model_path(root, "/wiki", "add", word)
                return '<a href="%s">%s</a>' % (add_url, word)

        content = wikiwords.sub(check, content)

        page_data = {
            'context': context,
            'content': content,
            'edit_link': context.get_absolute_url() + "/edit",
            'root': root,
        }
        return render_to_response("page.html", page_data)

    def edit(self, request, context, root, subpath, traversed):
        context_path = models.model_path(context, prepend="/wiki")

        if request.method == "POST":
            form = PageEditForm(request.POST)
            if form.is_valid():
                context.content = form.cleaned_data['content']
                return HttpResponseRedirect(context_path)
        else:
            form = PageEditForm(initial={'content': context.content})

        page_data = {
            'name': context.name,
            'context': context,
            'cancel_link': context_path,
            'form': form,
        }
        return render_to_response("edit.html", page_data)
views.registry.register(model=Page, view=PageView())


def create_frontpage(root):
    frontpage = Page()
    root["FrontPage"] = frontpage
    return root


def page(request, path):
    root = models.get_root(Wiki, setup=create_frontpage)
    return views.get_response_or_404(request, root=root, path=path)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from relstorage.adapters.oracle import OracleAdapter
from relstorage.options import Options

from django_zodb.config import parse_bool
from django_zodb.storage.rdbms import RelStorageFactory


class OracleFactory(RelStorageFactory):
    _adapter = OracleAdapter
    _adapter_args = (
        ('user', str, 'user'),
        ('password', str, 'password'),
        ('dsn', str, 'dsn'),
        ('twophase', parse_bool, 'twophase'),
    )

    def get_adapter(self, options):
        options = Options(**options)
        settings = self.config.get_settings(self._adapter_args)
        return self._adapter(options=options, **settings)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import re
from urlparse import parse_qs

# http://stackoverflow.com/questions/1175208/
FIRST_CAP_RE = re.compile('(.)([A-Z][a-z]+)')
ALL_CAP_RE = re.compile('([a-z0-9])([A-Z])')


def camel_case_to_underline(name):
    conv1 = FIRST_CAP_RE.sub(r'\1_\2', name)
    return ALL_CAP_RE.sub(r'\1_\2', conv1).lower()


def parse_uri(uri):
    def _push(ret, key, buf, next_key=None):
        if not buf:
            return next_key or key, []
        ret[key] = ''.join(buf)
        return next_key or key, []

    ret = {}
    has_dslash = "://" in uri
    has_user = bool(re.search(':(//)?[^:@?/#&\s]+:?@', uri))
    if "#" in uri:
        uri, frag = uri.rsplit("#", 1)
        ret['frag'] = frag

    key = 'scheme'
    buf = []
    for character in uri:
        if character == ":":
            if key == 'scheme':
                key, buf = _push(ret, key, buf)
                if has_dslash:
                    key = 'dslash1'
                elif has_user:
                    key = 'user'
                else:
                    key = 'host'
                continue
            elif key == 'user':
                key, buf = _push(ret, key, buf, 'password')
                continue
            elif key == 'host':
                key, buf = _push(ret, key, buf, 'port')
                continue

        if character == "/":
            if key == 'dslash1':
                key = 'dslash2'
                continue
            if key == 'dslash2':
                if has_user:
                    key = 'user'
                else:
                    key = 'host'
                continue

            if key not in ['path', 'query']:
                key, buf = _push(ret, key, buf, 'path')

        if character == "@":
            if key in ['user', 'password']:
                key, buf = _push(ret, key, buf, 'host')
                continue

        if character == "?":
            if key != "query":
                key, buf = _push(ret, key, buf, 'query')
                continue

        buf.append(character)

    # append remaining chars in buffer
    if buf:
        ret[key] = ''.join(buf)

    # sanitize
    if 'port' in ret:
        ret['port'] = int(ret['port'])
    if 'query' in ret:
        ret['query'] = parse_qs(ret['query'], keep_blank_values=True)

    return ret


# Repoze.BFG url_quote()
always_safe = ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
               'abcdefghijklmnopqrstuvwxyz'
               '0123456789' '_.-')
_safemaps = {}
_must_quote = {}


def url_quote(url_orig, safe=''):
    cachekey = (safe, always_safe)
    try:
        safe_map = _safemaps[cachekey]
        if not _must_quote[cachekey].search(url_orig):
            return url_orig
    except KeyError:
        safe += always_safe
        _must_quote[cachekey] = re.compile(r'[^%s]' % safe)
        safe_map = {}
        for i in range(256):
            char = chr(i)
            safe_map[char] = (char in safe) and char or ('%%%02X' % i)
        _safemaps[cachekey] = safe_map
    res = [safe_map[char] for char in url_orig]
    return ''.join(res)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from django.test import TestCase


class MiddlewareTests(TestCase):
    def setUp(self):
        import django.conf
        if hasattr(django.conf.settings, 'ZODB'):
            del django.conf.settings._wrapped.ZODB
        django.conf.settings._wrapped.ZODB = {'default': ['mem://']}

    def test_commit_enabled(self):
        import transaction
        from django_zodb.models import get_root
        from samples.wiki.models import Wiki

        response = self.client.post("/wiki/FrontPage/edit", { 'content': 'value_commited' }, follow=True)
        transaction.abort() # force removal of uncommited changes

        self.assertEquals('value_commited', get_root(Wiki)['FrontPage'].content)

    def test_commit_disabled(self):
        import transaction
        from django_zodb.models import get_root
        from samples.wiki.models import Wiki
        from django_zodb.middleware import TransactionMiddleware

        TransactionMiddleware.disable()
        response = self.client.post("/wiki/FrontPage/edit", { 'content': 'value_uncommited' }, follow=True)
        transaction.abort() # force removal of uncommited changes
        TransactionMiddleware.enable()

        self.assertNotEquals('value_uncommited', get_root(Wiki)['FrontPage'].content)





# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from django.test import TestCase

from django_zodb import models

class FakeContainer(models.Container):
    def set(self, key, value):
        self[key] = value
        value.__name__ = key
        value.__parent__ = self

class FakeModel(models.Model):
    pass

ROOT = FakeContainer()
ROOT.set('foo', FakeContainer())
ROOT['foo']['bar'] = FakeContainer()
ROOT['foo']['qux'] = FakeContainer()
ROOT['foo']['qux']['quxx'] = FakeModel()
ROOT.set(u'úñíçõdê', FakeContainer())

from django_zodb import models

class MyRoot(models.Root):
    def __init__(self, attr):
        self.attr = attr

class ModelsTests(TestCase):
    def eq(self, a, b, *args, **kwargs):
        return self.assertEquals(a, b, *args, **kwargs)

    def raise_(self, err, func, *args, **kwargs):
        return self.assertRaises(err, func, *args, **kwargs)

    def _set_zodb(self, dic):
        import django.conf
        django.conf.settings._wrapped.ZODB = dic

    def test_model_path(self):
        self.assertEquals(models.model_path(ROOT['foo']['qux']['quxx']), "/foo/qux/quxx")
        self.assertEquals(models.model_path(ROOT), u"/")
        self.assertEquals(models.model_path(ROOT[u'úñíçõdê']), u"/%C3%BA%C3%B1%C3%AD%C3%A7%C3%B5d%C3%AA")
        self.assertEquals(models.model_path(ROOT['foo'], prepend="x"), u"x/foo")

    def test_root(self):
        self._set_zodb({"default": ["mem://"]})

        root1 = models.get_root(MyRoot, attr="1")
        root2 = models.get_root(MyRoot, attr="2")

        self.assertEquals(id(root1), id(root2))
        self.assertEquals(root1.attr, root2.attr)
        self.assertEquals(root1.__name__, None)
        self.assertEquals(root1.__parent__, None)

    def test_remove_model(self):
        container = FakeContainer()

        ROOT['new'] = container
        self.assertEquals(ROOT['new'], container)
        self.assertEquals(container.__parent__, ROOT)

        del ROOT['new']
        self.assertEquals(container.__parent__, None)

    def test_invalid_root(self):
        self._set_zodb({"default": ["mem://"]})

        class InvalidRoot(object):
            pass

        self.assertRaises(TypeError, models.get_root, InvalidRoot)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import os

from django.test import TestCase

from testutils.tools import remove_db_files

from django_zodb.storage import factories

class StorageTests(TestCase):
    def setUp(self):
        remove_db_files()

    def tearDown(self):
        remove_db_files()

    def test_fail_unknown_scheme(self):
        from django_zodb.storage import get_storage_from_uri
        self.assertRaises(ValueError, get_storage_from_uri, 'unknown:')

    def test_disabled_reason(self):
        factories.disable("disabled", "reason")
        self.assertEquals(factories.disable_reason("disabled"), "reason")
        del factories.disabled['disabled'] # cleanup

    def test_mem_storage(self):
        from django_zodb.storage import get_storage_from_uri
        storage = get_storage_from_uri("mem://")
        self.assertEquals(storage.__class__.__name__, "MappingStorage")
        storage.close()

    def test_mem_storage_demo(self):
        from django_zodb.storage import get_storage_from_uri
        storage = get_storage_from_uri("mem://?demostorage")
        self.assertEquals(storage.__class__.__name__, "DemoStorage")
        storage.close()

    def test_file_storage(self):
        from django_zodb.storage import get_storage_from_uri
        storage = get_storage_from_uri("file:///tmp/test.db?database_name=file")
        self.assertEquals(storage.__class__.__name__, "FileStorage")
        self.assertRaises(TypeError, lambda: storage.fshelper.temp_dir)
        storage.close()
        remove_db_files()

    def test_fail_file_storage_with_invalid_arguments_1(self):
        from django_zodb.storage import get_storage_from_uri
        self.assertRaises(ValueError, get_storage_from_uri, "file:///tmp/test.db?create=true&read_only=true")

    def test_fail_file_storage_with_invalid_arguments_2(self):
        from django_zodb.storage import get_storage_from_uri
        self.assertRaises(TypeError, get_storage_from_uri, "file://?error=1")

    def test_fail_file_storage_with_invalid_arguments_3(self):
        from django_zodb.storage import get_storage_from_uri
        self.assertRaises(ValueError, get_storage_from_uri, "file:///foo/bar?path=error")

    def test_fail_file_storage_with_invalid_arguments_4(self):
        from django_zodb.storage import get_storage_from_uri
        self.assertRaises(TypeError, get_storage_from_uri, "file:///tmp/test.db?quota=not_int")

    def test_file_storage_with_blob(self):
        from django_zodb.storage import get_storage_from_uri
        storage = get_storage_from_uri("file:///tmp/test.db?blob_dir=/tmp/blobdir")
        self.assertEquals(storage.__class__.__name__, "FileStorage")
        self.assertEquals(storage.getName(), "/tmp/test.db")
        self.assertEquals(storage.fshelper.temp_dir, "/tmp/blobdir/tmp")
        storage.close()

        self.assertTrue(os.path.isdir('/tmp/blobdir'))
        self.assertTrue(os.path.isdir('/tmp/blobdir/tmp'))
        self.assertTrue(os.path.exists('/tmp/blobdir/.layout'))
        self.assertEqual(open('/tmp/blobdir/.layout').read().strip(), 'bushy')

    def _fake_factories(self, uri):
        from django_zodb.config import get_configuration_from_uri
        from django_zodb.storage import factories

        config = get_configuration_from_uri(uri)
        scheme = config.pop("scheme")
        factory_class = factories.get(scheme)

        ret = {}
        def _fake_adapter(self, **kwargs):
            del kwargs['options']
            ret['adapter'] = kwargs
            return "FakeAdapter"
        def _fake_storage(self, **kwargs):
            kwargs.pop('adapter', None)
            ret['storage'] = kwargs
            return "FakeStorage"
        factory_class._adapter = _fake_adapter
        factory_class._storage = _fake_storage

        factory_class(config).get_storage()

        return ret

    def test_zeo_storage_with_host(self):
        uri = "zeo://localhost/ignored_path?blob_dir=/tmp/blobdir&wait=true&wait_timeout=1"
        self.assertEquals(self._fake_factories(uri), {
            'storage': {
                'addr': ('localhost', 8090),
                'blob_dir': '/tmp/blobdir',
                'wait_timeout': 1,
                'wait': True,
            },
        })

    def test_zeo_storage_with_sock(self):
        uri = "zeo:///tmp/zeo.zdsock?blob_dir=/tmp/blobdir&wait=true&wait_timeout=1"
        self.assertEquals(self._fake_factories(uri), {
            'storage': {
                'addr': "/tmp/zeo.zdsock",
                'blob_dir': "/tmp/blobdir",
                'wait_timeout': 1,
                'wait': True,
            },
        })

    def test_fail_zeo_storage_without_host_and_sock(self):
        self.assertRaises(ValueError, self._fake_factories, "zeo://")

    def _enabled(self, scheme):
        enabled = scheme in factories.available()
        if not enabled:
            msg = "%s test disabled. Reason: %s" % (scheme, factories.disable_reason(scheme))
            import warnings
            warnings.warn(msg, UserWarning)
        return enabled

    def test_mysql_storage(self):
        if not self._enabled('mysql'):
            return

        uri = "mysql://"\
              "test_user:test_pass@"\
              "test_host"\
              "?compress=1&create=false&read_only=true&blob_dir=/tmp"\
              "&cache_servers=cache1,cache2,cache3"\
              "#test_dbname"

        self.assertEquals(self._fake_factories(uri), {
            'adapter': {
                'user':     'test_user',
                'passwd':   'test_pass',
                'host':     'test_host',
                'db':       'test_dbname',
                'compress': True,
            },
            'storage': {
                'read_only': True,
                'blob_dir':  '/tmp',
                'create':    False,
                'cache_servers': ('cache1', 'cache2', 'cache3'),
            },
        })

    def test_postgresql_storage(self):
        if not self._enabled('postgresql'):
            return

        uri = "postgresql://"\
              "test_user:test_pass@"\
              "test_host"\
              "?create=false"\
              "#test_dbname"

        self.assertEquals(self._fake_factories(uri), {
            'adapter': {
                'dsn': 'dbname=test_dbname host=test_host password=test_pass user=test_user',
            },
            'storage': {
                'create':    False,
            },
        })

    # def test_oracle_storage(self):
    #     if not self._enabled('oracle'):
    #         return
    #
    #     uri = "oracle://"\
    #           "test_user:test_pass@"\
    #           "?dsn=user1/pass1@dsn&twophase=true"
    #
    #     self.assertEquals(self._fake_factories(uri), {
    #         'adapter': {
    #             'user': 'test_user',
    #             'password': 'test_pass',
    #             'dsn': 'user1/pass1@dsn',
    #             'twophase': True,
    #         },
    #         'storage': {
    #             'create': True,
    #         },
    #     })



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from django.conf.urls.defaults import *

urlpatterns = patterns('',
    (r'^wiki/', include('samples.wiki.urls')),
)




# -*- coding: utf}-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from cStringIO import StringIO

from ZODB.DB import DB
import ZConfig

from django_zodb.config import get_configuration_from_uri, REQUIRED
from django_zodb.storage import get_storage


class DatabaseError(Exception):
    pass


class DatabaseFactory(object):
    _args = (
        ('database_name', str, 'database_name'),
        ('connection_cache_size', int, 'cache_size'),
        ('connection_pool_size', int, 'pool_size'),
    )
    _zconfig_args = (
        ('path', str, 'path', REQUIRED),
        ('frag', str, 'frag'),
    )
    _schema_xml_template = """<schema>
        <import package="ZODB"/>
        <multisection type="ZODB.database" attribute="databases" />
    </schema>"""

    def __init__(self, config):
        self.config = config

    def _get_database_from_zconfig(self):
        settings = self.config.get_settings(self._zconfig_args)

        path = settings['path']
        frag = settings.get('frag', '')

        schema = ZConfig.loadSchemaFile(StringIO(self._schema_xml_template))
        config, _ = ZConfig.loadConfig(schema, path)
        for database in config.databases:
            if not frag or frag == database.name:
                return database.open()
        else:
            raise ValueError("Database %r not found." % frag)

    def __call__(self):
        if self.config.get('scheme') == 'zconfig':
            return self._get_database_from_zconfig()

        settings = self.config.get_settings(self._args)
        storage = get_storage(self.config)
        return DB(storage, **settings)


def get_database_from_uris(uris):
    databases = {}
    ret = None
    for uri in uris:
        config = get_configuration_from_uri(uri)
        db_factory = DatabaseFactory(config)
        db = db_factory()
        for name in db.databases:
            if name in databases:
                raise ValueError("database_name %r already in databases." % name)
        databases.update(db.databases)
        db.databases = databases
        if ret is None:
            ret = db
    return ret


def get_database_by_name(name):
    from django.conf import settings
    if not hasattr(settings, 'ZODB') or not settings.ZODB:
        raise ValueError("Missing or empty settings.ZODB configuration.")

    try:
        return get_database_from_uris(settings.ZODB[name])
    except KeyError:
        raise ValueError("Database %r not found in settings.ZODB." % name)


class _ConnectionPool(object):
    def __init__(self):
        self._connections = {}

    def get_connection(self, name):
        if name not in self._connections:
            database = get_database_by_name(name)
            self._connections[name] = database.open()
        return self._connections[name]

connections = _ConnectionPool()


def get_connection(name):
    return connections.get_connection(name)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import logging

from ZODB.DemoStorage import DemoStorage

from django_zodb.config import parse_bool, get_configuration_from_uri


# Utilities
log = logging.getLogger("django_zodb.storage")


# Factory Base
class AbstractStorageFactory(object):
    _storage_args = ()

    def __init__(self, config):
        self.demostorage = parse_bool(config.pop('demostorage', "false"))
        self.config = config

    def get_base_storage(self, *args, **kwargs):
        raise NotImplementedError("Abstract class: %r, %r" % (args, kwargs))  # pragma: no cover abstract method code

    def get_storage(self):
        settings = self.config.get_settings(self._storage_args)
        storage = self.get_base_storage(**settings)
        if self.demostorage:
            storage = DemoStorage(base=storage)
        return storage


# Factories Registry
class _FactoriesRegistry(object):
    def __init__(self):
        self.enabled = {}
        self.disabled = {}

    def enable(self, scheme, factory_class):
        self.enabled[scheme] = factory_class

    def disable(self, scheme, reason):
        self.disabled[scheme] = str(reason)

    def available(self):
        return self.enabled.keys()

    def get(self, scheme):
        return self.enabled[scheme]

    def disable_reason(self, scheme):
        return self.disabled[scheme]

    def register(self, scheme, factory):
        try:
            package_name, module_name, factory_name = factory.rsplit(".", 2)
            module = __import__(package_name + "." + module_name, fromlist=module_name)
            factory_class = getattr(module, factory_name)
            self.enable(scheme, factory_class)
        except (ImportError, AttributeError, IndexError), ex:
            self.disable(scheme, ex)

factories = _FactoriesRegistry()


# Register Factories
factories.register("mem", "django_zodb.storage.base.MemoryFactory")
factories.register("file", "django_zodb.storage.base.FileFactory")
factories.register("zeo", "django_zodb.storage.zeo.ZEOFactory")
factories.register("mysql", "django_zodb.storage.mysql.MySQLFactory")
factories.register("postgresql", "django_zodb.storage.postgresql.PostgreSQLFactory")
factories.register("oracle", "django_zodb.storage.oracle.OracleFactory")


# Get storages
def get_storage(config):
    try:
        scheme = config.pop('scheme')
        factory = factories.get(scheme)
    except KeyError:
        raise ValueError('Invalid or Unknown scheme: %s' % (scheme,))

    storage = factory(config).get_storage()

    return storage


def get_storage_from_uri(uri):
    config = get_configuration_from_uri(uri)
    return get_storage(config)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import urllib

from django.http import Http404


# Exceptions
class TraversalError(Exception):
    pass


class MethodNotFound(TraversalError):
    pass


class ViewNotFound(TraversalError):
    pass


class _ViewRegistry(object):
    def __init__(self):
        self.views = {}

    def clean(self):
        self.views = {}

    def register(self, model, view):
        self.views[model] = view

    def get_view(self, context):
        try:
            return self.views[context.__class__]
        except KeyError:
            raise ViewNotFound("No registered view for %r" % context)

registry = _ViewRegistry()


class TraverseResult(object):
    def __init__(self, **kwargs):
        self._results = {
            'root': None,
            'context': None,
            'method_name': u"",
            'subpath': (),
            'traversed': (),
        }
        self._results.update(kwargs)

    def __setattr__(self, attr, value):
        if attr == "_results":
            super(TraverseResult, self).__setattr__(attr, value)
        else:
            self._results[attr] = value

    def __getattr__(self, attr):
        try:
            return self._results[attr]
        except KeyError:
            return getattr(self._results, attr)

    def __setitem__(self, key, value):
        self._results[key] = value

    def __getitem__(self, key):
        return self._results[key]


def split_path(path):
    path = path.strip('/')
    clean = []
    for segment in path.split('/'):
        segment = urllib.unquote(segment)
        if not segment or segment == '.':
            continue
        elif segment == '..':
            clean.pop()
        else:
            try:
                clean.append(segment.decode('utf-8'))
            except UnicodeDecodeError:
                raise TypeError('Could not decode path segment %r using the '
                                'UTF-8 decoding scheme' % segment)
    return tuple(clean)


def traverse(root, path):
    path = split_path(path)

    context = root
    i = 0
    for i, segment in enumerate(path):
        if segment.startswith('@@'):
            segment = segment[2:]
            break

        try:
            getitem = context.__getitem__
        except AttributeError:
            break

        try:
            context = getitem(segment)
        except (TypeError, IndexError, KeyError):
            break
    else:
        segment = u""

    result = TraverseResult(
        context=context,
        method_name=segment,
        subpath=path[i + 1:],
        traversed=path[:i + 1],
        root=root,
    )
    return result


def get_response(request, root, path):
    result = traverse(root, path)
    view = registry.get_view(result.context)
    return view(request, result)


def get_response_or_404(request, root, path):
    try:
        return get_response(request, root, path)
    except TraversalError, ex:
        raise Http404("%r not found (%s)." % (path, ex))


class View(object):
    def __call__(self, request, result):
        method_name = result.pop("method_name") or "__index__"
        try:
            method = getattr(self, method_name)
        except AttributeError:
            raise MethodNotFound("View %r does not exist." % method_name)
        return method(request, **result)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from test_utils import URIParseTests
from test_storage import StorageTests
from test_database import DatabaseTests
from test_views import ViewsTests
from test_models import ModelsTests
from test_middleware import MiddlewareTests



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import code
from django.core.management.base import NoArgsCommand
from django_zodb.database import get_database_by_name


class Command(NoArgsCommand):
    help = "Runs a Python interactive interpreter and automatically import get_database_by_name() function."
    requires_model_validation = False

    def handle_noargs(self, **options):
        imported_objects = {'get_database_by_name': get_database_by_name}
        code.interact(local=imported_objects)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from django.test import TestCase


from testutils.tools import get_tool_path, turn_off_log


turn_off_log("ZODB.FileStorage")
turn_off_log("ZEO.zrpc")

class DatabaseTests(TestCase):
    def _clean_settings(self):
        import django.conf
        if hasattr(django.conf.settings, 'ZODB'):
            del django.conf.settings._wrapped.ZODB

    def _set_zodb(self, dic):
        import django.conf
        django.conf.settings._wrapped.ZODB = dic

    def test_fail_unknown_scheme(self):
        from django_zodb.database import get_database_from_uris
        self.assertRaises(ValueError, get_database_from_uris, ['unknown:'])

    def test_mem_database(self):
        from django_zodb.database import get_database_from_uris
        db = get_database_from_uris(["mem://"])
        self.assertEquals("unnamed", db.database_name)
        self.assertTrue("unnamed" in db.databases)
        self.assertEquals("MappingStorage", db.getName())

    def test_mem_demo_database(self):
        from django_zodb.database import get_database_from_uris
        db = get_database_from_uris(["mem://?demostorage=true"])
        self.assertEquals("unnamed", db.database_name)
        self.assertTrue("unnamed" in db.databases)
        self.assertEquals("DemoStorage('MappingStorage', 'MappingStorage')", db.getName())

    def test_fail_multi_db_same_name(self):
        from django_zodb.database import get_database_from_uris
        self.assertRaises(ValueError, get_database_from_uris, ["mem://", "mem://"])

    def test_multi_db(self):
        from django_zodb.database import get_database_from_uris
        db = get_database_from_uris(["mem://", "mem://?database_name=catalog"])
        self.assertEquals("unnamed", db.database_name)
        self.assertTrue("unnamed" in db.databases)
        self.assertTrue("catalog" in db.databases)
        self.assertEquals("MappingStorage", db.getName())

    def test_zconfig_database_1(self):
        from django_zodb.database import get_database_from_uris
        db = get_database_from_uris(["zconfig://" + get_tool_path('zconfig.zcml') + "#temp2"])
        self.assertFalse("temp1" in db.databases)
        self.assertTrue("temp2" in db.databases)

    def test_zconfig_database_2(self):
        from django_zodb.database import get_database_from_uris
        db = get_database_from_uris(["zconfig://" + get_tool_path('zconfig.zcml')])
        self.assertTrue("temp1" in db.databases)
        self.assertFalse("temp2" in db.databases)

    def test_fail_zconfig_unknown_database(self):
        from django_zodb.database import get_database_from_uris
        uri = "zconfig://" + get_tool_path('zconfig.zcml') + "#unknown"
        self.assertRaises(ValueError, get_database_from_uris, [uri])

    def test_fail_not_configured_db_1(self):
        self._clean_settings()

        from django_zodb.database import get_database_by_name
        self.assertRaises(ValueError, get_database_by_name, 'db1')

        self._set_zodb({})
        self.assertRaises(ValueError, get_database_by_name, 'db1')

        self._set_zodb({'db1': ['mem://']})
        self.assertRaises(ValueError, get_database_by_name, 'db2')

    def test_open_file_database(self):
        self._clean_settings()
        self._set_zodb({'db1': ['file:///tmp/test.db']})

        from django_zodb.database import get_database_by_name
        db = get_database_by_name('db1')
        self.assertEquals("unnamed", db.database_name)
        self.assertTrue("unnamed" in db.databases)
        self.assertEquals("/tmp/test.db", db.getName())



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import transaction

from persistent import Persistent
from persistent.mapping import PersistentMapping

from django_zodb.utils import url_quote, camel_case_to_underline
from django_zodb.database import get_connection


class BaseRoot(type):
    def __new__(mcs, name, bases, attrs):
        new_class = super(BaseRoot, mcs).__new__(mcs, name, bases, attrs)

        parents = [b for b in bases if isinstance(b, BaseRoot)]
        if not parents:
            return new_class

        meta = attrs.pop("Meta", None)
        new_class.__database__ = getattr(meta, "database", "default")
        new_class.__rootname__ = getattr(meta, "rootname", camel_case_to_underline(name))
        return new_class


class Model(Persistent):
    __name__ = None
    __parent__ = None


class Root(Model):
    __metaclass__ = BaseRoot


class Container(PersistentMapping):
    __name__ = None
    __parent__ = None

    def __setitem__(self, key, value):
        PersistentMapping.__setitem__(self, key, value)
        value.__parent__ = self
        value.__name__ = key

    def __delitem__(self, key):
        self[key].__parent__ = None
        PersistentMapping.__delitem__(self, key)


class RootContainer(Container):
    __metaclass__ = BaseRoot


def _setuproot(root):
    return root


def get_root(rootclass, setup=_setuproot, commit=True, *args, **kwargs):
    try:
        database = rootclass.__database__
        rootname = rootclass.__rootname__
    except AttributeError:
        raise TypeError("%r must implements Root interface." % (rootclass,))

    zodbroot = get_connection(database).root()

    if rootname not in zodbroot:
        zodbroot[rootname] = setup(rootclass(*args, **kwargs))
        if commit:
            transaction.commit()

    return zodbroot[rootname]


def _lineage(model):
    while model is not None:
        yield model
        try:
            model = model.__parent__
        except AttributeError:
            model = None


def _quote_segment(segment):
    if segment.__class__ is unicode:  # isinstance slighly slower (~15%)
        return url_quote(segment.encode('utf-8'))
    return url_quote(segment)


def model_path(model, prepend=None, *elements):
    path = [location.__name__ or u'' for location in _lineage(model)]
    path.reverse()
    path.extend(elements)

    if any(path):
        ret = u'/'.join(_quote_segment(segment) for segment in path)
    else:
        ret = u'/'

    if prepend is not None:
        return unicode(prepend) + ret

    return ret



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from relstorage.adapters.mysql import MySQLAdapter
from relstorage.options import Options

from django_zodb.config import parse_bool
from django_zodb.storage.rdbms import RelStorageFactory


class MySQLFactory(RelStorageFactory):
    _adapter = MySQLAdapter
    _adapter_args = (
        ('host', str, 'host'),
        ('user', str, 'user'),
        ('password', str, 'passwd'),
        ('frag', str, 'db'),
        ('port', int, 'port'),
        ('path', str, 'unix_socket'),
        ('connect_timeout', int, 'connect_timeout'),
        ('compress', parse_bool, 'compress'),
        ('named_pipe', parse_bool, 'named_pipe'),
        ('init_command', str, 'init_command'),
        ('read_default_file', str, 'read_default_file'),
        ('read_default_group', str, 'read_default_group'),
        ('client_flag', int, 'client_flag'),
        ('load_infile', int, 'load_infile'),
    )

    def get_adapter(self, options):
        options = Options(**options)
        settings = self.config.get_settings(self._adapter_args)
        return self._adapter(options=options, **settings)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from django.test import TestCase

class URIParseTests(TestCase):
    def _t(self, uri, **kwargs):
        from django_zodb.utils import parse_uri
        self.assertEquals(parse_uri(uri), kwargs)

    def test_uris(self):
        self._t("sc", scheme='sc')
        self._t("sc:", scheme='sc')
        self._t("sc://", scheme='sc')
        self._t("sc:host", scheme='sc', host='host')
        self._t("sc://host", scheme='sc', host='host')
        self._t("sc://user@", scheme='sc', user='user')
        self._t("sc:user@", scheme='sc', user='user')
        self._t("sc://user@host", scheme='sc', user='user', host='host')
        self._t("sc://user:pass@host", scheme='sc', user='user', password='pass',
                    host='host')
        self._t("sc:user:pass@host", scheme='sc', user='user', password='pass',
                    host='host')
        self._t("sc://user:pass@host:1234", scheme='sc', user='user', password='pass',
                    host='host', port=1234)
        self._t("sc://user:pass@host:1234/", scheme='sc', user='user', password='pass',
                    host='host', port=1234, path='/')
        self._t("sc://user:pass@host:1234/path", scheme='sc', user='user', password='pass',
                    host='host', port=1234, path='/path')
        self._t("sc://user:pass@host/path", scheme='sc', user='user', password='pass',
                    host='host', path='/path')
        self._t("sc://user:pass@host///path", scheme='sc', user='user', password='pass',
                    host='host', path='///path')
        self._t("sc://user:pass@host/path/", scheme='sc', user='user', password='pass',
                    host='host', path='/path/')

        self._t("cache://user:pass@host/path/", scheme='cache', user='user', password='pass',
                    host='host', path='/path/')
        self._t("cache://user:pass@host/path/", scheme='cache', user='user', password='pass',
                    host='host', path='/path/')

        self._t("file:///foo", scheme='file', path="/foo")
        self._t("file://user@/foo", user='user', scheme='file', path="/foo")
        self._t("file://foo/bar", scheme='file', host="foo", path="/bar")
        self._t("file:///foo@bar&baz&qux.quxx/bla:ble", scheme='file', path="/foo@bar&baz&qux.quxx/bla:ble")
        self._t("file:///foo@bar&baz&qux.quxx?bla=ble&bli", scheme='file', path="/foo@bar&baz&qux.quxx",
                    query={'bla': ['ble'], 'bli': ['']})
        self._t("file:///foo#bar/baz#frag", scheme="file", path="/foo#bar/baz", frag="frag")

        self._t("sc?foo", scheme='sc', query={"foo": ['']})
        self._t("sc:?foo", scheme='sc', query={"foo": ['']})
        self._t("sc:?foo=bar", scheme='sc', query={"foo": ['bar']})
        self._t("sc:?foo=bar&bar=baz", scheme='sc', query={"foo": ['bar'], "bar": ['baz']})
        self._t("sc:?foo=bar&foo=bar", scheme='sc', query={"foo": ['bar', 'bar']})
        self._t("sc://u:p@h:1/x?a=b",
                    scheme='sc',
                    user='u',
                    password='p',
                    host='h',
                    port=1,
                    path='/x',
                    query={'a': ['b']}
                    )
        self._t("sc://u@h:1/x?a=b",
                    scheme='sc',
                    user='u',
                    host='h',
                    port=1,
                    path='/x',
                    query={'a': ['b']}
                    )
        self._t("sc://u@:1/x?a=b",
                    scheme='sc',
                    user='u',
                    port=1,
                    path='/x',
                    query={'a': ['b']}
                    )
        self._t("sc://h:1/x?a=b", scheme='sc', host='h', port=1, path='/x', query={'a': ['b']})
        self._t("sc://:1/x?a=b", scheme='sc', port=1, path='/x', query={'a': ['b']})
        self._t("sc:///x?a=b", scheme='sc', path='/x', query={'a': ['b']})
        self._t("sc://?a=b", scheme='sc', query={'a': ['b']})
        self._t("mysql://test_user:test_pass@test_host?compress=1#test_dbname",
            scheme="mysql",
            user="test_user",
            password="test_pass",
            host="test_host",
            query={'compress': ['1']},
            frag="test_dbname",
        )

    def test_camel_case_to_underline(self):
        from django_zodb.utils import camel_case_to_underline as cc2u

        self.assertEquals(cc2u("Foo"), "foo")
        self.assertEquals(cc2u("foo"), "foo")
        self.assertEquals(cc2u("FooBar"), "foo_bar")
        self.assertEquals(cc2u("FooBarBaz"), "foo_bar_baz")
        self.assertEquals(cc2u("FooBarBazX"), "foo_bar_baz_x")
        self.assertEquals(cc2u("ISBNNumber"), "isbn_number")
        self.assertEquals(cc2u("XMod1"), "x_mod1")
        self.assertEquals(cc2u("Convert2ISBN"), "convert2_isbn")
        self.assertEquals(cc2u("Convert_2_ISBN"), "convert_2_isbn")



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from relstorage.storage import RelStorage

from django_zodb.storage.base import AbstractStorageFactory
from django_zodb.config import parse_bool, parse_tuple


class RelStorageFactory(AbstractStorageFactory):
    _storage = RelStorage
    _storage_args = (
        ('name', str, 'name'),
        ('read_only', parse_bool, 'read_only'),
        ('blob_dir', str, 'blob_dir'),
        ('shared_blob_dir', bool, 'shared_blob_dir'),
        ('blob_cache_size', int, 'blob_cache_size'),
        ('blob_cache_size_check', int, 'blob_cache_size_check'),
        ('blob_cache_chunk_size', int, 'blob_cache_chunk_size'),
        ('keep_history', parse_bool, 'keep_history'),
        ('replica_conf', str, 'replica_conf'),
        ('replica_timeout', float, 'replica_timeout'),
        ('poll_interval', int, 'poll_interval'),
        ('pack_gc', parse_bool, 'pack_gc'),
        ('pack_dry_run', parse_bool, 'pack_dry_run'),
        ('pack_batch_timeout', float, 'pack_batch_timeout'),
        ('pack_duty_cycle', float, 'pack_duty_cycle'),
        ('pack_max_delay', float, 'pack_max_delay'),
        ('cache_servers', parse_tuple, 'cache_servers'),
        ('cache_module_name', str, 'cache_module_name'),
        ('cache_prefix', str, 'cache_prefix'),
        ('cache_local_mb', int, 'cache_local_mb'),
        ('cache_delta_size_limit', int, 'cache_delta_size_limit'),
        ('commit_lock_timeout', int, 'commit_lock_timeout'),
        ('commit_lock_id', int, 'commit_lock_id'),
        ('strict_tpc', bool, 'strict_tpc'),
        ('create', parse_bool, 'create'),
    )

    def get_adapter(self, *args, **kwargs):
        raise NotImplementedError("Abstract class: %r, %r" % (args, kwargs))  # pragma: no cover abstract method code

    def get_base_storage(self, **options):
        create = options.pop("create", True)  # HACK: missing in relstorage.options.Options(?)
        adapter = self.get_adapter(options)
        options['create'] = create
        options['adapter'] = adapter
        return self._storage(**options)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from relstorage.adapters.postgresql import PostgreSQLAdapter
from relstorage.options import Options

from django_zodb.storage.rdbms import RelStorageFactory


class PostgreSQLFactory(RelStorageFactory):
    _adapter = PostgreSQLAdapter
    _adapter_args = (
        ('frag', str, 'dbname'),
        ('host', str, 'host'),
        ('port', int, 'port'),
        ('user', str, 'user'),
        ('password', str, 'password'),
        ('sslmode', str, 'sslmode'),
        ('connect_timeout', int, 'connect_timeout'),
    )

    def get_adapter(self, options):
        options = Options(**options)
        settings = self.config.get_settings(self._adapter_args)
        args = []
        for key in sorted(settings.keys()):
            args.append("%s=%s" % (key, settings[key]))
        dsn = " ".join(args)
        return self._adapter(dsn=dsn, options=options)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import markdown  # http://pypi.python.org/pypi/Markdown

from django_zodb import models


# models.RootContainer - Define a 'root' object for database. This class
#                        defines __parent__ = __name__ = None
class Wiki(models.RootContainer):
    def pages(self):
        for pagename in sorted(self):
            yield self[pagename]

    def get_absolute_url(self):
        return "/wiki"

    # It's possible to change models.RootContainer settings using Meta
    # configurations. Here we will explicitly define the default values
    class Meta:
        database = 'default'  # Optional. Default: 'default'
        rootname = 'wiki'     # Optional. Default: RootClass.__name__.lower()


# models.Container - We will use Container to add support to subpages.
class Page(models.Model):
    def __init__(self, content="Empty Page."):
        super(Page, self).__init__()
        self.content = content

    def html(self):
        md = markdown.Markdown(
                safe_mode="escape",
                extensions=('codehilite', 'def_list', 'fenced_code'))
        return md.convert(self.content)

    @property
    def name(self):
        return self.__name__

    def get_absolute_url(self):
        return u"/".join((self.__parent__.get_absolute_url(), self.name))



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = ':memory:'

ZODB = { 'default': ['mem:'], }

ROOT_URLCONF = 'urls'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django_zodb.middleware.TransactionMiddleware',
)

TEMPLATE_DIRS = (os.path.join(ROOTDIR, "templates"),)

INSTALLED_APPS = (
    'django_zodb',
    'tests',
    'samples.wiki',
)

# Test settings
TEST_RUNNER='testutils.django_coverage.test_runner_with_coverage'
COVERAGE_MODULES = (
    'django_zodb.storage',
    'django_zodb.storage.base',
    'django_zodb.storage.rdbms',
    'django_zodb.storage.mysql',
    'django_zodb.storage.postgresql',
    'django_zodb.database',
    'django_zodb.utils',
    'django_zodb.views',
    'django_zodb.models',
    'django_zodb.config',
    'django_zodb.middleware',
)




# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

import os

import sys
sys.path.append("src")

from setuptools import setup, find_packages

from django_zodb.version import __version__

long_description = file(
    os.path.join(
        os.path.dirname(__file__),
            'README.rst'
        )
).read()

setup(
    name='django-zodb',
    version=__version__,
    description='Using Django and ZODB together',
    long_description=long_description,
    author='Osvaldo Santana Neto',
    author_email='osantana@triveos.com',
    license="BSD",
    url='http://triveos.github.com/django-zodb/',
    download_url='http://github.com/triveos/django-zodb/downloads',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=[
        'distribute',
        'ZODB3>=3.10.0a2',
        'Django>=1.2.3',
    ],
    extras_require={
        'MySQL': [
            'RelStorage>=1.4.0b3',
            'MySQL-python>=1.2.3c1',
        ],
        'PostgreSQL': [
            'RelStorage>=1.4.0b3',
            'psycopg2>=2.2.0rc1',
        ],
        'Oracle': [
            'RelStorage>=1.4.0b3',
            'cx_Oracle>=5.0.3',
        ],
        'samples': [
            'markdown',
        ],
    },
    classifiers=[
        'Environment :: Web Environment',
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)




# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#
# Empty file just to make Django happy (and run the tests).



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import os
ROOTDIR = os.path.dirname(os.path.realpath(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = ':memory:'

ZODB = {
    'default': ['file://' + os.path.join(ROOTDIR, "wiki_db.fs")],
}

ROOT_URLCONF = 'samples.urls'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django_zodb.middleware.TransactionMiddleware',
)

TEMPLATE_DIRS = (os.path.join(ROOTDIR, "templates"),)

INSTALLED_APPS = (
    'django_zodb',
    'samples.wiki',
)





# -*- coding: utf-8 -*-
#
# django-zodb sample - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


from django import forms


class PageEditForm(forms.Form):
    content = forms.CharField(widget=forms.Textarea)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#


import os

from ZODB.MappingStorage import MappingStorage
from ZODB.blob import BlobStorage
from ZODB.FileStorage.FileStorage import FileStorage

from django_zodb.config import parse_bool, REQUIRED
from django_zodb.storage import AbstractStorageFactory


# Basic Storage Factories
class MemoryFactory(AbstractStorageFactory):
    _storage = MappingStorage
    _storage_args = (
        ('blob_dir', str, 'base_directory'),
    )

    def get_base_storage(self, **kwargs):
        storage = self._storage()
        if 'base_directory' in kwargs:
            storage = BlobStorage(storage=storage, **kwargs)
        return storage


class FileFactory(AbstractStorageFactory):
    _storage = FileStorage
    _storage_args = (
        ('path', os.path.normpath, 'file_name', REQUIRED),
        ('create', parse_bool, 'create'),
        ('read_only', parse_bool, 'read_only'),
        ('quota', int, 'quota'),
        ('blob_dir', str, 'blob_dir'),
    )

    def get_base_storage(self, **kwargs):
        return self._storage(**kwargs)



# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

import transaction


class TransactionMiddleware(object):
    enabled = True

    def process_exception(self, request, exception):  # pylint:disable-msg=W0613
        if self.enabled:
            transaction.abort()

    def process_response(self, request, response):  # pylint:disable-msg=W0613
        if self.enabled:
            transaction.commit()
        return response

    @classmethod
    def disable(cls):
        cls.enabled = False

    @classmethod
    def enable(cls):
        cls.enabled = True
